<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;


use Ems\Services\SolicitacaoService;

class SolicitacaoController {

    /** @var OcorrenciaService */
    protected $solicitacaoService;
    
    public function __construct(
        EntityManager $entityManager,
        SolicitacaoService $solicitacaoService
    ) {
        $this->entityManager        = $entityManager;
        $this->solicitacaoService   = $solicitacaoService;
    }


    public function getSolicitacoesVinculosGestor(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrEstrutura = (int) $row['NRESTRUTURAM'];

            $solicitacoes = $this->solicitacaoService->
                getSolicitacoesVinculosGestor($nrEstrutura);

            $response->addDataSet(new DataSet('getSolicitacoesVinculosGestor', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
    
    public function getItemSolicitacao(Request\Row $request, Response $response) {
        try {
            $params = $request->getRow();

            $nrsolicitacao = $params['NRSOLICITACAO'];

            $solicitacoes = $this->solicitacaoService->getItemSolicitacao($nrsolicitacao);

            $response->addDataSet(new DataSet('getItemSolicitacao', $solicitacoes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function getTiposolicitacao(Request\Row $request, Response $response) {
        try {

            $solicitacoes = $this->solicitacaoService->getTiposolicitacao();

            $response->addDataSet(new DataSet('getTiposolicitacao', $solicitacoes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getTipoRelacionamento(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposRel = $this->solicitacaoService->getTipoRelacionamento();

            $response->addDataSet(new DataSet('getTipoRelacionamento', $tiposRel));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getLinhaTransporteAtiva(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $linhas = $this->solicitacaoService->getLinhaTransporteAtiva();

            $response->addDataSet(new DataSet('getLinhaTransporteAtiva', $linhas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipolinhavinc(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposLinhas = $this->solicitacaoService->getTipolinhavinc();

            $response->addDataSet(new DataSet('getTipolinhavinc', $tiposLinhas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getIntegtransp(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $numerosIntegracao = $this->solicitacaoService->getIntegtransp();

            $response->addDataSet(new DataSet('getIntegtransp', $numerosIntegracao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getSentidolinha(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $sentidos = $this->solicitacaoService->getSentidolinha();

            $response->addDataSet(new DataSet('getSentidolinha', $sentidos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getBeneficio(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $beneficios = $this->solicitacaoService->getBeneficio();

            $response->addDataSet(new DataSet('getBeneficio', $beneficios));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getPessoaRelacionamentos(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrPessoa = (int) $row['NRPESSOA'];
            
            $pessoas = $this->solicitacaoService->getPessoaRelacionamentos($nrPessoa);

            $response->addDataSet(new DataSet('getPessoaRelacionamentos', $pessoas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}
