<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

use Util\DateUtil;

class AdmissaoRepository {
    /** @var EntityManager */
    protected $entityManager;

    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->entityManager        = $entityManager;
        $this->connection           = $entityManager->getConnection();
    }

    public function getVinculosNaoAprovados(array $params): array {
        $sql = <<<SQL
            SELECT /*+ ordered*/
                VM.*,
                PH.*,
                PH.CDPAIS AS CDPAISNATURALI,
                PH.SGESTADO AS SGESTADONATURALI,
                PH.CDMUNICIPIO AS CDMUNICIPIONATURALI,
                VH.NRVINCULOH,
                VH.NRSITUFUNCM,
                VH.NRCARGO,
                VH.NRESCALATRABM,
                VH.DTMESCOMPETENC,
                VH.IDCONTRIBUISIND,
                VH.NRDEPENDIR,
                VH.NRDEPENDSFAM,
                VH.DTBASEFERIAS,
                VH.IDREMUNERACAO,
                VH.IDTPPAGAMENTO,
                VH.NRESTRUTLEGAL,
                VH.NRESTRUTGEREN,
                VH.NRTPMODALIDSAL,
                VH.IDMULTVINC,
                VH.NRFUNCAO,
                VH.CDCONTRIBINDIVIDUAL,
                VH.NRTPMOVTRANSFM,
                VH.DTFIMCONTRDETERMIN,
                VH.NRESTRUTSIND,
                VH.CONFIDENCIAL,
                VH.NRTURNO,
                VH.NRTPREGIMEPREV,
                NVL(VH.IDPAGPROPFER13, 'N') IDPAGPROPFER13,
                FPA_TPADMISSAO.NMTPADMISSAO,
                ESTRUTLEGAL.NMESTRUTURAM AS NMESTRUTLEGAL,
                ESTRUTGEREN.NMESTRUTURAM AS NMESTRUTGEREN,
                ESTRUTSIND.NMESTRUTURAM AS NMESTRUTSIND,
                TVH.NMTIPOVINCULOH,
                VE.NMVINCULOEMPREG,
                VE.NMABREV,
                TMT.NMTPMOVTRANSFH,
                RP.DSTPREGIMEPREV,
                CARGO.NMOCUPACAOH AS NMCARGO,
                FUNCAO.NMOCUPACAOH AS NMFUNCAO,
                CBO.NRCBO,
                CBO.NMCBO,
                GPE_ALTEESCALA.DTINIESCALA,
                GPE_ALTEESCALA.DTFIMESCALA,
                GPE_ALTEESCALA.QTHRLIMITESEPHEXTRA,
                ESC.NMESCALATRABH,
                GPE_TURNO.DSTURNO,
                SH.NMSITUFUNCH,
                TMS.NMTPMODALIDSAL,
                GPE_ALTESALARIO.IDTPSALARIO,
                GPE_ALTESALARIO.VRSALARIO,
                GPE_ALTESALARIO.NROCORRENCIA,
                GPE_TIPOSALARIO.DSTIPOSALARIO,
                ESTRG.NMESTADO AS NMUFRGPESSOA,
                ESTCTPS.NMESTADO AS NMUFCTPSPES,
                CONTCORRPARC.NRCONTCORRPARC,
                CONTCORRPARC.CDBANCO,
                CONTCORRPARC.CDAGENCIA,
                CONTCORRPARC.CDCONTCORR,
                CONTCORRPARC.NRTPCONTCORR,
                BANCO.NMBANCO,
                AGENCIA.NMAGENCIA,
                TPCONTCORR.DSTPCONTCORR,
                TELPRINCIPAL_.CDPREFIXCOMUPARC PREFTELPRINCIPAL,
                TELPRINCIPAL_.CDCOMUNICAPARC TELPRINCIPAL,
                TELRESIDEN_.CDPREFIXCOMUPARC PREFTELRESIDEN,
                TELRESIDEN_.CDCOMUNICAPARC TELRESIDEN,
                TELCOMERC_.CDPREFIXCOMUPARC PREFTELCOMERC,
                TELCOMERC_.CDCOMUNICAPARC TELCOMERC,
                CELPARTIC_.CDPREFIXCOMUPARC PREFCELPARTIC,
                CELPARTIC_.CDCOMUNICAPARC CELPARTIC,
                CELCOMERC_.CDPREFIXCOMUPARC PREFCELCOMERC,
                CELCOMERC_.CDCOMUNICAPARC CELCOMERC,
                FAX_.CDPREFIXCOMUPARC PREFFAX,
                FAX_.CDCOMUNICAPARC FAX,
                EMAIL_.CDCOMUNICAPARC EMAIL,
                ENDE.NRENDERECOPARC,
                ENDE.CDLOGRADOURO,
                ENDE.DSENDERECO,
                ENDE.NRIMOVELENDERECO,
                ENDE.NMBAIRROENDERECO,
                ENDE.NRCEPENDERECO,
                ENDE.SGESTADO,
                ENDE.CDPAIS,
                ENDE.CDMUNICIPIO,
                ENDE.DSCOMPLEENDERECO,
                ENDE.DSREFERENCIAENDE,
                ENDE.CDTIPOENDERECO,
                MUN.NMMUNICIPIO,
                PAIS.NMPAIS,
                CLEST.DSCLASSESTRANG
            FROM
                GPE_PESSOA PE
                JOIN
                GPE_PESSOAH PH ON
                    PH.NRPESSOA = PE.NRPESSOA AND
                    PH.NRORG =  PE.NRORG
                LEFT JOIN
                GPE_VINCULOM VM ON
                    PE.NRPESSOA = VM.NRPESSOA AND
                    PE.NRORG = VM.NRORG
                LEFT JOIN
                GPE_VINCULOH VH ON
                    VH.NRVINCULOM = VM.NRVINCULOM AND
                    VH.NRORG = VM.NRORG
                LEFT JOIN
                GPE_TIPOVINCULOH TVH ON
                    TVH.NRTIPOVINCULOM = VM.NRTIPOVINCULOM AND
                    (
                        TVH.NRORG = VM.NRORG OR
                        TVH.NRORG =:P_NRORG_PADRAO
                    ) AND
                    TVH.DTMESCOMPETENC = (
                        SELECT
                            MAX(TVHH.DTMESCOMPETENC)
                        FROM
                            GPE_TIPOVINCULOH TVHH
                        WHERE
                            TVHH.NRORG = TVH.NRORG AND
                            TVHH.NRTIPOVINCULOM = TVH.NRTIPOVINCULOM AND
                            TVHH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                GPE_TPMOVTRANSFH TMT ON
                    TMT.NRTPMOVTRANSFM = VH.NRTPMOVTRANSFM AND
                    (
                        TMT.NRORG = VH.NRORG OR
                        TMT.NRORG =:P_NRORG_PADRAO
                    ) AND
                    TMT.DTMESCOMPETENC = (
                        SELECT
                            MAX(TMTH.DTMESCOMPETENC)
                        FROM
                            GPE_TPMOVTRANSFH TMTH
                        WHERE
                            TMTH.NRORG = TMT.NRORG AND
                            TMTH.NRTPMOVTRANSFM = TMT.NRTPMOVTRANSFM AND
                            TMTH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                GPE_OCUPACAOH CARGO ON
                    CARGO.NROCUPACAOM = VH.NRCARGO AND
                    CARGO.NRORG = VH.NRORG AND
                    CARGO.DTMESCOMPETENC = (
                        SELECT
                            MAX(CARGOH.DTMESCOMPETENC)
                        FROM
                            GPE_OCUPACAOH CARGOH
                        WHERE
                            CARGOH.NRORG = CARGO.NRORG AND
                            CARGOH.NROCUPACAOM = CARGO.NROCUPACAOM AND
                            CARGOH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                GPE_OCUPACAOH FUNCAO ON
                    FUNCAO.NROCUPACAOM = VH.NRFUNCAO AND
                    FUNCAO.NRORG = VH.NRORG AND
                    FUNCAO.DTMESCOMPETENC = (
                        SELECT
                            MAX(FUNCAOH.DTMESCOMPETENC)
                        FROM
                            GPE_OCUPACAOH FUNCAOH
                        WHERE
                            FUNCAOH.NRORG = FUNCAO.NRORG AND
                            FUNCAOH.NROCUPACAOM = FUNCAO.NROCUPACAOM AND
                            FUNCAOH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                GPE_CBO CBO ON
                    FUNCAO.NRCBO = CBO.NRCBO
                LEFT JOIN
                ESTRUTURAM ESTRUTLEGAL ON
                    ESTRUTLEGAL.NRESTRUTURAM = VH.NRESTRUTLEGAL AND
                    ESTRUTLEGAL.NRORG = VH.NRORG
                LEFT JOIN
                ESTRUTURAM ESTRUTGEREN ON
                    ESTRUTGEREN.NRESTRUTURAM = VH.NRESTRUTGEREN AND
                    ESTRUTGEREN.NRORG = VH.NRORG
                LEFT JOIN
                ESTRUTURAM ESTRUTSIND ON
                    ESTRUTSIND.NRESTRUTURAM = VH.NRESTRUTSIND AND
                    ESTRUTSIND.NRORG = VH.NRORG
                LEFT JOIN
                ESTADO ESTRG ON
                    ESTRG.SGESTADO = PH.SGUFRGPESSOA AND
                    ESTRG.CDPAIS = '0055'
                LEFT JOIN
                ESTADO ESTCTPS ON
                    ESTCTPS.SGESTADO = PH.SGUFCTPSPES AND
                    ESTCTPS.CDPAIS = '0055'
                LEFT JOIN
                GPE_VINCULOEMPREG VE ON
                    VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
                LEFT JOIN
                FPA_TPMODALIDSAL TMS ON
                    TMS.NRTPMODALIDSAL = VH.NRTPMODALIDSAL AND
                    (
                        VH.NRORG = TMS.NRORG OR
                        TMS.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                GPE_ALTESALARIO ON
                    VM.NRVINCULOM = GPE_ALTESALARIO.NRVINCULOM AND
                    VM.NRORG = GPE_ALTESALARIO.NRORG
                LEFT JOIN
                GPE_TIPOSALARIO ON
                    GPE_TIPOSALARIO.CDTIPOSALARIO = GPE_ALTESALARIO.IDTPSALARIO
                LEFT JOIN
                GPE_TPREGIMEPREV RP ON
                    RP.NRTPREGIMEPREV = VH.NRTPREGIMEPREV AND
                    (
                        VH.NRORG = RP.NRORG OR
                        RP.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                GPE_ALTEESCALA ON
                    GPE_ALTEESCALA.NRESCALATRABM = VH.NRESCALATRABM AND
                    GPE_ALTEESCALA.NRVINCULOM = VM.NRVINCULOM AND
                    GPE_ALTEESCALA.NRORG = VH.NRORG
                LEFT JOIN
                GPE_ESCALATRABH ESC ON
                    ESC.NRESCALATRABM = GPE_ALTEESCALA.NRESCALATRABM AND
                    ESC.NRORG = VH.NRORG AND
                    ESC.DTMESCOMPETENC = (
                        SELECT
                            MAX(ESCH.DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH ESCH
                        WHERE
                            ESCH.NRORG = ESC.NRORG AND
                            ESCH.NRESCALATRABM = ESC.NRESCALATRABM AND
                            ESCH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )
                LEFT JOIN
                GPE_TURNO ON
                    GPE_TURNO.NRTURNO = GPE_ALTEESCALA.NRTURNO AND
                    GPE_TURNO.NRORG = GPE_ALTEESCALA.NRORG
                LEFT JOIN
                GPE_SITUFUNCH SH ON
                    (
                        SH.NRORG = VH.NRORG OR
                        SH.NRORG =:P_NRORG_PADRAO
                    ) AND
                    SH.NRSITUFUNCM = VH.NRSITUFUNCM AND
                    SH.DTEFETIVA = (
                        SELECT
                            MAX(SHH.DTEFETIVA)
                        FROM
                            GPE_SITUFUNCH SHH
                        WHERE
                            SHH.NRORG = SH.NRORG AND
                            SHH.NRSITUFUNCM = SH.NRSITUFUNCM AND
                            DTEFETIVA <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                CONTCORRPARC ON
                        CONTCORRPARC.NRPARCNEGOCIO = PE.NRPARCNEGOCIO AND
                        CONTCORRPARC.NRORG = PE.NRORG
                LEFT JOIN
                BANCO ON
                    BANCO.CDBANCO = CONTCORRPARC.CDBANCO AND
                    (
                        BANCO.NRORG = CONTCORRPARC.NRORG OR
                        BANCO.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                TPCONTCORR ON
                    TPCONTCORR.NRTPCONTCORR = CONTCORRPARC.NRTPCONTCORR AND
                    (
                        TPCONTCORR.NRORG = CONTCORRPARC.NRORG OR
                        TPCONTCORR.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                AGENCIA ON
                    CONTCORRPARC.CDAGENCIA = AGENCIA.CDAGENCIA AND
                    CONTCORRPARC.CDBANCO = AGENCIA.CDBANCO AND
                    (
                        AGENCIA.NRORG = CONTCORRPARC.NRORG OR
                        AGENCIA.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                COMUNICAPARC TELPRINCIPAL_ ON
                    PE.NRPARCNEGOCIO = TELPRINCIPAL_.NRPARCNEGOCIO AND
                    PE.NRORG = TELPRINCIPAL_.NRORG AND
                    TELPRINCIPAL_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = TELPRINCIPAL_.NRPARCNEGOCIO AND
                            CP.NRORG = TELPRINCIPAL_.NRORG AND
                            CP.CDFORMACOMU = '01'
                    )
                LEFT JOIN
                COMUNICAPARC TELRESIDEN_ ON
                    PE.NRPARCNEGOCIO = TELRESIDEN_.NRPARCNEGOCIO AND
                    PE.NRORG = TELRESIDEN_.NRORG AND
                    TELRESIDEN_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = TELRESIDEN_.NRPARCNEGOCIO AND
                            CP.NRORG = TELRESIDEN_.NRORG AND
                            CP.CDFORMACOMU = '03'
                        )
                LEFT JOIN
                COMUNICAPARC TELCOMERC_ ON
                    PE.NRPARCNEGOCIO = TELCOMERC_.NRPARCNEGOCIO AND
                    PE.NRORG = TELCOMERC_.NRORG AND
                    TELCOMERC_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = TELCOMERC_.NRPARCNEGOCIO AND
                            CP.NRORG = TELCOMERC_.NRORG AND
                            CP.CDFORMACOMU = '04'
                        )
                LEFT JOIN
                COMUNICAPARC CELPARTIC_ ON
                    PE.NRPARCNEGOCIO = CELPARTIC_.NRPARCNEGOCIO AND
                    PE.NRORG = CELPARTIC_.NRORG AND
                    CELPARTIC_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = CELPARTIC_.NRPARCNEGOCIO AND
                            CP.NRORG = CELPARTIC_.NRORG AND
                            CP.CDFORMACOMU = '02'
                        )
                LEFT JOIN
                COMUNICAPARC CELCOMERC_ ON
                    PE.NRPARCNEGOCIO = CELCOMERC_.NRPARCNEGOCIO AND
                    PE.NRORG = CELCOMERC_.NRORG AND
                    CELCOMERC_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = CELCOMERC_.NRPARCNEGOCIO AND
                            CP.NRORG = CELCOMERC_.NRORG AND
                            CP.CDFORMACOMU = '06'
                        )
                LEFT JOIN
                COMUNICAPARC FAX_ ON
                    PE.NRPARCNEGOCIO = FAX_.NRPARCNEGOCIO AND
                    PE.NRORG = FAX_.NRORG AND
                    FAX_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = FAX_.NRPARCNEGOCIO AND
                            CP.NRORG = FAX_.NRORG AND
                            CP.CDFORMACOMU = '10'
                        )
                LEFT JOIN
                COMUNICAPARC EMAIL_ ON
                    PE.NRPARCNEGOCIO = EMAIL_.NRPARCNEGOCIO AND
                    PE.NRORG = EMAIL_.NRORG AND
                    EMAIL_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = EMAIL_.NRPARCNEGOCIO AND
                            CP.NRORG = EMAIL_.NRORG AND
                            CP.CDFORMACOMU = '09'
                        )
                LEFT JOIN
                ENDERECOPARC ENDE ON
                    PE.NRPARCNEGOCIO = ENDE.NRPARCNEGOCIO AND
                    PE.NRORG = ENDE.NRORG AND
                    ENDE.NRENDERECOPARC = (
                        SELECT
                            MAX(ENDER.NRENDERECOPARC)
                        FROM
                            ENDERECOPARC ENDER
                        WHERE
                            ENDER.NRPARCNEGOCIO = ENDE.NRPARCNEGOCIO AND
                            ENDER.NRORG = ENDE.NRORG AND
                            ENDER.CDTIPOENDERECO = 'PRINCIPAL'
                        )
                LEFT JOIN
                PAIS ON
                    PAIS.CDPAIS = ENDE.CDPAIS
                LEFT JOIN
                MUNICIPIO MUN ON
                    MUN.CDMUNICIPIO = ENDE.CDMUNICIPIO AND
                    MUN.CDPAIS = ENDE.CDPAIS AND
                    ENDE.SGESTADO = MUN.SGESTADO
                LEFT JOIN
                FPA_TPADMISSAO ON
                    FPA_TPADMISSAO.NRTPADMISSAO = VM.NRTPADMISSAO AND
                    (
                        FPA_TPADMISSAO.NRORG = VM.NRORG OR
                        FPA_TPADMISSAO.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                GPE_CLASSESTRANG CLEST ON
                    CLEST.NRCLASSESTRANG = PH.NRCLASSESTRANG
            WHERE
                PE.NRORG =:P_NRORG AND
                (
                    VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC) OR
                    VM.NRVINCULOM IS NULL
                ) AND
                (
                    VM.DTRESCISAOVINC IS NULL OR
                    VM.DTRESCISAOVINC >=:P_DTMESCOMPETENC
                ) AND
                (
                    (INSTR(:P_NRESTRUTURAS, ',' || VH.NRESTRUTGEREN || ',')) > 0 OR
                    :P_NRESTRUTURAS IS NULL
                ) AND (
                    (
                        TO_CHAR(PH.DTMESCOMPETENC, 'DD/MM/YYYY') = '01/01/3000' AND
                        VM.NRVINCULOM IS NULL
                    ) OR
                    (
                        TO_CHAR(VH.DTMESCOMPETENC, 'DD/MM/YYYY') = '01/01/3000' AND
                        VM.NRVINCULOM IS NOT NULL
                    )
                ) AND
                (
                    (
                        PH.DTMESCOMPETENC = (
                            SELECT
                                MAX(PHH.DTMESCOMPETENC)
                            FROM GPE_PESSOAH PHH
                            WHERE
                                PHH.NRORG = PH.NRORG AND
                                PHH.NRPESSOA = PH.NRPESSOA AND
                                PHH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                        ) AND
                        VM.NRVINCULOM IS NOT NULL
                    ) OR
                    TO_CHAR(PH.DTMESCOMPETENC, 'DD/MM/YYYY') = '01/01/3000' AND
                    VM.NRVINCULOM IS NULL
                )
            ORDER BY PH.NMPESSOA
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }
}