<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class SolicitacaoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function getSolicitacoesVinculosGestor(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                SL.NRORG,
                SL.NRSOLICITACAO,
                SL.NRTIPOSOLICITACAO,
                SL.NRVINCULOM,
                TO_CHAR(SL.DTSOLICITACAO,'DD/MM/YYYY') AS DTSOLICITACAO,
                SL.IDAPROVACAOPRI,
                DECODE (SL.IDAPROVACAOPRI,  'A', 'Aprovada',
                                            'R', 'Rejeitada',
                                            'P', 'Pendente',
                                            'Pendente') DSAPROVACAOPRI,
                DECODE (SL.IDAPROVACAOSEC,  'A', 'Aprovada',
                                            'R', 'Rejeitada',
                                            'P', 'Pendente',
                                            'Pendente') DSAPROVACAOSEC,
                SL.DTAPROVACAOPRI,
                SL.CDOPERAPROVPRI,
                SL.IDAPROVACAOSEC,
                SL.DTAPROVACAOSEC,
                SL.CDOPERAPROVSEC,
                SL.DSOBSSOLICITACAO,
                SL.DSOBSAPROVAPRI,
                SL.DSOBSAPROVASEC,
                VM_S.NMVINCULOM,
                TP.NMTIPOSOLICITACAO,
                PS.NRNIVELAPROVASOLI,
                PS.NRPERFACESAPROVPRI,
                PS.NRPERFACESAPROVSEC
            FROM
                GPE_SOLICITACAO SL
                LEFT JOIN
                GPE_TIPOSOLICITACAO TP
                    ON SL.NRTIPOSOLICITACAO = TP.NRTIPOSOLICITACAO
                LEFT JOIN
                GPE_PERFACESTIPOSOLI PS
                    ON PS.NRORG = SL.NRORG
                   AND PS.NRTIPOSOLICITACAO = SL.NRTIPOSOLICITACAO
                LEFT JOIN
                GPE_VINCULOM VM_S
                    ON VM_S.NRORG = SL.NRORG
                   AND VM_S.NRVINCULOM = SL.NRVINCULOM
                LEFT JOIN
                GPE_VINCULOH VH_S
                    ON VH_S.NRORG = VM_S.NRORG
                   AND VH_S.NRVINCULOM = VM_S.NRVINCULOM
                   AND VH_S.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_VINCULOH V1
                        WHERE
                            V1.NRORG = VH_S.NRORG
                        AND V1.NRVINCULOM = VH_S.NRVINCULOM
                        AND V1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
            WHERE
                (
                    SL.NRORG = :P_NRORG
                    OR SL.NRORG = 0
                )
            AND (INSTR(:P_NRESTRUTURAS, ',' || VH_S.NRESTRUTGEREN || ',')) > 0
            ORDER BY NMVINCULOM ASC, SL.DTSOLICITACAO ASC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $solicitacoes = $std->fetchAll();

        return $solicitacoes;
    }

    public function getItemSolicitacao($nrorg, $nrsolicitacao ) {
        $sql = <<<SQL
    SELECT
        IT.NRORG,
        IT.NRITEMSOLICITACAO,
        IT.NRSOLICITACAO,
        IT.DSCHAVEITEMSOLI,
        IT.DSLABELITEMSOLI,
        IT.DSVALORITEMSOLI,
        IT.DTINCLUSAO,
        IT.DTULTATU,
        IT.NRORGINCLUSAO,
        IT.CDOPERINCLUSAO,
        IT.NRORGULTATU,
        IT.CDOPERULTATU,
        TP.NMTIPOSOLICITACAO
    FROM GPE_ITEMSOLICITACAO IT
    JOIN
          GPE_SOLICITACAO SL
          ON SL.NRSOLICITACAO = IT.NRSOLICITACAO
          AND SL.NRORG = IT.NRORG

    LEFT JOIN
          GPE_TIPOSOLICITACAO TP
          ON SL.NRTIPOSOLICITACAO = TP.NRTIPOSOLICITACAO

    WHERE
        IT.NRORG = :P_NRORG
        AND IT.NRSOLICITACAO = :P_NRSOLICITACAO
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRSOLICITACAO'   => $nrsolicitacao
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }


    public function getTiposolicitacao($nrorg) {
        $sql = <<<SQL
    SELECT
   *
    FROM
       GPE_TIPOSOLICITACAO
    WHERE
       (
          NRORG = :P_NRORG
          OR NRORG = 0
       )
        AND NRTIPOSOLICITACAO IN (3, 5, 9, 10, 11, 13, 14)
    ORDER BY
        NMTIPOSOLICITACAO
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg
            )
        );

        $resultQuery = $query->fetchAll();


        if (isset($resultQuery)) {
            return $resultQuery;
        } else {
            return null;
        }
    }
    
    public function getSolicitacao($nrorg, $nrsolicitacao = null) {
        $sql = <<<SQL
    SELECT
        *
    FROM
        GPE_SOLICITACAO SL
    JOIN GPE_TIPOSOLICITACAO TS ON (TS.NRORG = SL.NRORG OR TS.NRORG = 0) AND TS.NRTIPOSOLICITACAO = SL.NRTIPOSOLICITACAO
    WHERE
        SL.NRORG = :P_NRORG
        AND (SL.NRSOLICITACAO = :P_NRSOLICITACAO OR :P_NRSOLICITACAO IS NULL)
SQL;


        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRSOLICITACAO'   => $nrsolicitacao,
            )
        );

        $resultQuery = empty($nrsolicitacao) ? $query->fetchAll() : $query->fetch();
        
        return $resultQuery;
    }

    public function getTipoRelacionamento(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                TIPORELACIONA
            WHERE
                NRORG = :P_NRORG OR
                NRORG = :P_NRORG_PADRAO
            ORDER BY
                NMTIPORELACIONA ASC
SQL;


        $std = $this->connection->prepare($sql);

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];

        $std->execute($params);
        $tiposRel = $std->fetchAll();

        return $tiposRel;
    }

    public function getLinhaTransporteAtiva(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                L.*,
                F.DSFORMATRANSPORTE,
                O.DSOPERTRANSPORTE,
                C.DSCATEGTRANSPORTE,
                T.DSTIPOTRANSPORTE,
                VLT.NRLINHATARIFA,
                VT.VRTARIFA
            FROM
                VAB_LINHA L
                LEFT JOIN
                FORMATRANSPORTE F ON
                    F.NRFORMATRANSPORTE = L.NRFORMATRANSPORTE AND
                    (
                        F.NRORG = L.NRORG OR
                        F.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                OPERTRANSPORTE O ON
                    O.NROPERTRANSPORTE = L.NROPERTRANSPORTE AND
                    (
                        O.NRORG = L.NRORG OR
                        O.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                CATEGTRANSPORTE C ON
                    C.NRCATEGTRANSPORTE = L.NRCATEGTRANSPORTE AND
                    (
                        C.NRORG = L.NRORG OR
                        C.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                TIPOTRANSPORTE T ON
                T.NRTIPOTRANSPORTE = L.NRTIPOTRANSPORTE AND
                (
                    T.NRORG = L.NRORG OR
                    T.NRORG =:P_NRORG_PADRAO
                )
                LEFT JOIN
                VAB_LINHATARIFA VLT ON
                    L.NRLINHATRANSP = VLT.NRLINHATRANSP AND
                    L.NRORG = VLT.NRORG AND
                    (
                        VLT.DTFIMLINHATARIFA > LAST_DAY(:P_DTMESCOMPETENC) OR
                        VLT.DTFIMLINHATARIFA IS NULL
                    )
                    LEFT JOIN
                    VAB_TARIFA VT ON
                    VLT.NRTARIFA = VT.NRTARIFA AND
                    (
                        VT.NRORG = :P_NRORG OR
                        VT.NRORG =:P_NRORG_PADRAO
                    )
            WHERE
                (
                    L.NRORG = :P_NRORG OR
                    L.NRORG =:P_NRORG_PADRAO
                ) AND
                (
                    L.DTFIMLINHA > LAST_DAY(:P_DTMESCOMPETENC) OR
                    L.DTFIMLINHA IS NULL
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $linhas = $std->fetchAll();

        return $linhas;
    }

    public function getTipolinhavinc(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                TIPOLINHAVINC
            WHERE
                NRORG IN (:P_NRORG, :P_NRORGPADRAO) AND
                IDATIVO = 'S'
SQL;

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRORGPADRAO' => $nrOrgPadrao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tiposLinhas = $std->fetchAll();

        return $tiposLinhas;
    }

    public function getIntegtransp(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                VAB_INTEGTRANSPM.DTINIVALIDADEINTEG,
                VAB_INTEGTRANSPM.DTFINVALIDADEINTEG,
                VAB_INTEGTRANSPH.*,
                EM.NMESTRUTURAM,
                TIPOLINHAVINC.DSTIPOLINHAVINC,
                TP.DSTIPOINTEGTRANSP
            FROM
                VAB_INTEGTRANSPM
                JOIN
                VAB_INTEGTRANSPH ON
                    VAB_INTEGTRANSPH.NRORG = VAB_INTEGTRANSPM.NRORG AND
                    VAB_INTEGTRANSPH.NRINTEGTRANSPM = VAB_INTEGTRANSPM.NRINTEGTRANSPM
                LEFT JOIN
                TIPOLINHAVINC ON
                    TIPOLINHAVINC.NRTIPOLINHAVINC = VAB_INTEGTRANSPH.NRTIPOLINHAVINC
                LEFT JOIN
                ESTRUTURAM EM ON
                    EM.NRESTRUTURAM = VAB_INTEGTRANSPH.NRESTRUTURAM AND
                    EM.NRORG = VAB_INTEGTRANSPH.NRORG
                LEFT JOIN
                ESTRUTURAH EH ON
                    EH.NRESTRUTURAM = EM.NRESTRUTURAM AND
                    EH.NRORG = EM.NRORG AND
                    EH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            ESTRUTURAH
                        WHERE
                            NRORG = EH.NRORG AND
                            NRESTRUTURAM = EH.NRESTRUTURAM AND
                            DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
                LEFT JOIN
                TIPOINTEGTRANSP TP ON
                    (
                        TP.NRORG = VAB_INTEGTRANSPH.NRORG OR
                        TP.NRORG = :P_NRORG_PADRAO
                    ) AND
                    TP.NRTIPOINTEGTRANSP = VAB_INTEGTRANSPH.NRTIPOINTEGTRANSP
            WHERE
                VAB_INTEGTRANSPM.NRORG = :P_NRORG AND
                DTMESCOMPINTEG = (
                    SELECT
                        MAX(DTMESCOMPINTEG)
                    FROM
                        VAB_INTEGTRANSPH H1
                    WHERE
                        H1.NRORG = VAB_INTEGTRANSPH.NRORG AND
                        H1.NRINTEGTRANSPM = VAB_INTEGTRANSPH.NRINTEGTRANSPM AND
                        H1.DTMESCOMPINTEG <= :P_DTMESCOMPETENC
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tiposLinhas = $std->fetchAll();

        return $tiposLinhas;
    }

    public function getSentidolinha(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                SENTIDOLINHA
            WHERE
                NRORG IN (:P_NRORG, :P_NRORG_PADRAO)
SQL;

        $params = [
            'P_NRORG'       => $nrOrg,
            'P_NRORG_PADRAO' => $nrOrgPadrao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $sentidos = $std->fetchAll();

        return $sentidos;
    }

    public function getBeneficio(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                VAB_BENEFICIOM BM
                JOIN
                VAB_BENEFICIOH BH ON
                    BH.NRBENEFICIOM = BM.NRBENEFICIOM AND
                    BH.NRORG = BM.NRORG AND
                    BH.DTMESCOMPETENC = (
                        SELECT
                            MAX(BH1.DTMESCOMPETENC)
                        FROM
                            VAB_BENEFICIOH BH1
                        WHERE
                            BH1.NRBENEFICIOM = BM.NRBENEFICIOM AND
                            BH1.NRORG = BM.NRORG
                    )
            WHERE
                BM.NRORG IN (:P_NRORG, :P_NRORG_PADRAO)
SQL;

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        $beneficios = $std->fetchAll();
        
        return $beneficios;
    }
    
    public function getPessoaRelacionamentos(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                PRINCIPAL.*,
                (
                    CASE
                        WHEN NMTIPORELACIONA IS NULL THEN NMPESSOA
                        ELSE (NMPESSOA || ' - ' || NMTIPORELACIONA)
                    END
                ) DSRELACIONA
            FROM (
                SELECT
                    PE.NRPESSOA,
                    PH.NMPESSOA,
                    PH.NMPESSOA AS NMPESSOAPRINC,
                    PH.NRPESSOA AS NRPESSOAPRINC,
                    PH.DTNASCPESSOA AS DTNASCIMENTO,
                    PH.NRCPFPESSOA AS NRCPFPESSOA,
                    PE.NRPARCNEGOCIO,
                    NULL AS NRTIPORELACIONA,
                    '' AS NMTIPORELACIONA
                FROM
                    GPE_PESSOA PE     
                    JOIN
                    GPE_PESSOAH PH ON
                        PH.NRPESSOA = PE.NRPESSOA AND
                        PH.NRORG = PE.NRORG AND
                        PH.DTMESCOMPETENC = (
                            SELECT
                                MAX(DTMESCOMPETENC)                                                  
                            FROM
                                GPE_PESSOAH H1                                                  
                            WHERE
                                H1.NRPESSOA = PH.NRPESSOA AND
                                H1.NRORG = PH.NRORG AND
                                H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )     
                WHERE
                    PE.NRPESSOA = :P_NRPESSOA AND
                    PE.NRORG = :P_NRORG    
                
                UNION
                
                SELECT
                    RELAC.NRPESSOA,
                    RELACH.NMPESSOA,
                    PH.NMPESSOA AS NMPESSOAPRINC,
                    PH.NRPESSOA AS NRPESSOAPRINC,
                    RELACH.DTNASCPESSOA AS DTNASCIMENTO,
                    RELACH.NRCPFPESSOA AS NRCPFPESSOA,
                    RELAC.NRPARCNEGOCIO,
                    TR.NRTIPORELACIONA,
                    TR.NMTIPORELACIONA     
                FROM
                    GPE_PESSOA PRINC     
                    JOIN
                    GPE_PESSOAH PH ON
                        PH.NRPESSOA = PRINC.NRPESSOA AND
                        PH.NRORG = PRINC.NRORG AND
                        PH.DTMESCOMPETENC = (
                            SELECT
                                MAX(DTMESCOMPETENC)                                                  
                            FROM
                                GPE_PESSOAH H1                                                  
                            WHERE
                                H1.NRPESSOA = PH.NRPESSOA AND
                                H1.NRORG = PH.NRORG AND
                                H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )
                    JOIN
                    PARCNEGOCIO PCN ON
                        PCN.NRPARCNEGOCIO = PRINC.NRPARCNEGOCIO AND
                        PCN.NRORG = PRINC.NRORG     
                    JOIN
                    RELACIONAPARC RP ON
                        RP.NRPARCNEGOCIO = PRINC.NRPARCNEGOCIO AND
                        RP.NRORG = PRINC.NRORG     
                    JOIN
                    GPE_PESSOA RELAC ON
                        RELAC.NRPARCNEGOCIO = RP.NRPARCNEGRELAC AND
                        RELAC.NRORG = RP.NRORG     
                    JOIN
                    GPE_PESSOAH RELACH ON
                        RELACH.NRPESSOA = RELAC.NRPESSOA AND
                        RELACH.NRORG = RELAC.NRORG AND
                        RELACH.DTMESCOMPETENC =  (
                            SELECT
                                MAX(DTMESCOMPETENC)              
                            FROM
                                GPE_PESSOAH H1              
                            WHERE
                                H1.NRPESSOA = RELACH.NRPESSOA AND
                                H1.NRORG = RELACH.NRORG AND
                                H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )     
                    JOIN
                    TIPORELACIONA TR ON
                        TR.NRTIPORELACIONA = RP.NRTIPORELACIONA AND
                        (
                            TR.NRORG = RP.NRORG OR
                            TR.NRORG = :P_NRORGPADRAO
                        )     
                WHERE
                    PRINC.NRPESSOA = :P_NRPESSOA AND
                    (
                        INSTR(',' || :P_NRTIPORELACIONA || ',' , ',' || TR.NRTIPORELACIONA || ',') > 0 OR
                        :P_NRTIPORELACIONA IS NULL
                    ) AND
                    PRINC.NRORG = :P_NRORG
                ORDER BY
                    NMPESSOAPRINC,
                    NMPESSOA
            ) PRINCIPAL
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        $pessoas = $std->fetchAll();
        
        return $pessoas;
    }
}