<?php

namespace Ems\Util;

class Utils {
    
    const ORDEM_ASC     = 'ASC';
    const ORDEM_DESC    = 'DESC';
    
    const DEFAULT_FLOAT_DECIMAL_FIELDS_PRECISION = 3;
    
    public static function getIdDiaSemana(string $date): string {
        $formattedDate = Utils::formatDateYMD($date);

        $dayOfWeek = date('w', strtotime($formattedDate));

        $idRelation = [
            '1000000',
            '0100000',
            '0010000',
            '0001000',
            '0000100',
            '0000010',
            '0000001',
        ];

        $idDiaSemana = $idRelation[$dayOfWeek];

        return $idDiaSemana;
    }

    public static function formatDateYMD(string $date): string {
        [$day, $month, $year] = explode('/', $date);
        $formattedDate = "{$year}-{$month}-{$day}";

        return $formattedDate;
    }

    public static function formatDateDMY(string $date): string {
        [$year, $month, $day] = explode('-', $date);
        $formattedDate = "{$day}/{$month}/{$year}";

        return $formattedDate;
    }

    public static function stringToDateTime(string $date): \DateTimeImmutable {
        $formattedDate = Utils::formatDateYMD($date);
        $dateTime = new \DateTimeImmutable($formattedDate);

        return $dateTime;
    }

    public static function addDaysToDate(string $date, int $days): string {
        $dateTime = Utils::stringToDateTime($date);
        $interval = new \DateInterval("P{$days}D");
        $newDateTime = $dateTime->add($interval);
        $newDate = $newDateTime->format('d/m/Y');

        return $newDate;
    }

    public static function subDaysFromDate(string $date, int $days): string {
        $dateTime = Utils::stringToDateTime($date);
        $interval = new \DateInterval("P{$days}D");
        $newDateTime = $dateTime->sub($interval);
        $newDate = $newDateTime->format('d/m/Y');

        return $newDate;
    }
    
    public static function areFloatsEqual($a, $b, $precision = self::DEFAULT_FLOAT_DECIMAL_FIELDS_PRECISION) {
        return abs(($a-$b)/$b) < pow(10, -1 * $precision);
    }

    public static function getSqlDateTimeString(
        string $date,
        string $time
    ): string {
        $dateTime = "$date $time:00";

        return $dateTime;
    }

    /**
     * Preenche os campos especificados da row com o valor escolhido caso não esteja definido.
     *
     * @param array $row A row a ser preenchida.
     * @param string[] $fields Os campos que, caso não estejam definidos, serão preenchidos com o valor especificado.
     * @param mixed $value O valor que preencherá os campos especificados.
     *
     * @return array A row enviada com os campos preenchidos.
     */
    public static function fillRowWithValueIfUndefined(
        array $row,
        array $fields,
        $value = null
    ): array {
        foreach ($fields as $field) {
            $row[$field] = $row[$field] ?? $value;
        }

        return $row;
    }

    /**
     * Remove os pontos e vírgulas da string.
     *
     * @param string $str A string a ter a pontuação removida.
     *
     * @return string A string passado com a pontuação removida.
     */
    public static function removePunctuation(
        string $str
    ): string {
        $search = ['.', ',', '-'];
        $str = str_replace($search, '', $str);

        return $str;
    }
}